/*
 * File   : cdrom.c
 * Purpose: This file contains some routines to read information from
 *          a CD-ROM drive with an audio CD in it and routines to read
 *          a CDDB-Database.
 * Autor  : Andreas Theofilu
 *
 * Copyright (C) 1999 by Theos Soft.
 */

/* 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
/*
 * The following header file needs the library 'libcdaudio' to be
 * installed!
 */
#include <cdaudio.h>

#include <Xmt/Xmt.h>
#include <Xmt/Create.h>
#include <Xmt/Converters.h>
#include <Xmt/Procedures.h>
#include <Xmt/Layout.h>
#include <Xmt/Chooser.h>
#include <Xmt/InputField.h>
#include <Xmt/WidgetType.h>
#include <Xmt/Dialogs.h>

#include "platten.h"
#include "cdrom.h"

/*
 * Funktionen welche nur in diesem Modul verwendet werden.
 */
int CheckLibVersion ();
unsigned long cddbGetDiscID (struct disc_info *disc, char *device);

char *genre[] =
{
  "Blues",            "Classic Rock",         "Country",           "Dance",
  "Disco",            "Funk",                 "Grunge",            "Hip-Hop",
  "Jazz",             "Metal",                "New Age",           "Oldies",
  "Other",            "Pop",                  "R&B",               "Rap",
  "Reggae",           "Rock",                 "Techno",            "Industrial",
  "Alternative",      "Ska",                  "Death Metal",       "Pranks",
  "Soundtrack",       "Euro-Techno",          "Ambient",           "Trip-Hop",
  "Vocal",            "Jazz+Funk",            "Fusion",            "Trance",
  "Classical",        "Instrumental",         "Acid",              "House",
  "Game",             "Sound Clip",           "Gospel",            "Noise",       "AlternRock",       "Bass",                 "Soul",              "Punk",
  "Space",            "Meditative",           "Instrumental Pop",  "Instrumental Rock",
  "Ethnic",           "Gothic",               "Darkwave",          "Techno-Industrial",
  "Electronic",       "Pop-Folk",             "Eurodance",         "Dream",
  "Southern Rock",    "Comedy",               "Cult",              "Gangsta",
  "Top 40",           "Christian Rap",        "Pop/Funk",          "Jungle",
  "Native American",  "Cabaret",              "New Wave",          "Psychadelic",
  "Rave",             "Showtunes",            "Trailer",           "Lo-Fi",
  "Tribal",           "Acid Punk",            "Acid Jazz",         "Polka",
  "Retro",            "Musical",              "Rock & Roll",       "Hard Rock",
  "Folk",             "Folk-Rock",            "National Folk",     "Swing",
  "Fast Fusion",      "Bebob",                "Latin",             "Revival",
  "Celtic",           "Bluegrass",            "Avantgarde",        "Gothic Rock",
  "Progressive Rock", "Psychedelic Rock",     "Symphonic Rock",    "Slow Rock",
  "Big Band",         "Chorus",               "Easy Listening",    "Acoustic",
  "Humour",           "Speech",               "Chanson",           "Opera",
  "Chamber Music",    "Sonata",               "Symphony",          "Booty Bass",
  "Primus",           "Porn Groove",          "Satire",            "Slow Jam",
  "Club",             "Tango",                "Samba",             "Folklore",
  "Ballad",           "Power Ballad",         "Rhytmic Soul",      "Freestyle",
  "Duet",             "Punk Rock",            "Drum Solo",         "Acapella",
  "Euro-House",       "Dance Hall",           "Goa",               "Drum & Bass",
  "Club-House",       "Hardcore",             "Terror",            "Indie",
  "BritPop",          "Negerpunk",            "Polsk Punk",        "Beat",
  "Christian Gangsta Rap", "Heavy Metal",     "Black Metal",       "Crossover",
  "Contemporary Christian", "Christian Rock", "Merengue",          "Salsa",
  "Trash Metal",
  "Unknown"
};

unsigned int bitrates[3][3][15] =
{
  {
    {0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448},
    {0, 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384},
    {0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320}
  },
  {
    {0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256},
    {0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160},
    {0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160}
  },
  {
    {0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256},
    {0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160},
    {0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160}
  }
};

long s_freq[3][4] =
{
  {44100L, 48000L, 32000L, 0L},
  {22050L, 24000L, 16000L, 0L},
  {11025L, 8000L, 8000L, 0L}
};

int mask[8]={0x1, 0x2, 0x4, 0x8, 0x10, 0x20, 0x40, 0x80};

/*
 * Folgende Funktion liest die Informationen der angegebenen 'discid'
 * aus der CDDB-Datenbank. Damit auch die Spielzeiten der einzelnen
 * Tracks ermittelt werden koennen, mu die entsprechende CD eingelegt
 * sein. Die Routine ermittelt die DiscID direkt von der CD und
 * ergaenzt die fehlenden Daten.
 */
int cddbGetDiscInfo (cddb_title_t *cddb, unsigned long discid, char *device, char *path, char *category)
{
struct disc_info disc;
unsigned long cdid;
int fd, i;
char cdfile[256], hv0[256], *p;

	memset (cddb, 0, sizeof (cddb_title_t));
	cdid = cddbGetDiscID (&disc, device);

	if (cdid < 100L)
	   return (int)cdid;

	if (discid > 0L && cdid != discid)
	   return CD_EDISCID;

	/*
	 * Zusammensetzen des Pfades
	 */
	sprintf (&cdfile[0], "%s/%s/%08x", path, category, cdid);

	/*
	 * CDDB-Eintrag oeffnen
	 */
	if ((fd = open (cdfile, O_RDONLY, 0400)) == -1)
	   return CD_ECDDBEN;

	/*
	 * An dieser Stelle steht fest, das es sich um einen gueltigen
	 * CDDB-Eintrag handelt. Zuerst wird nun die DiscID zugewiesen,
	 * dann wird in einer Schleife die Datei ausgelesen und der
	 * Inhalt in die Struktur kopiert.
	 */
	cddb->discid = cdid;
	cddb->tracks = disc.disc_total_tracks;

	if (cddb->tracks > 30)
	   cddb->tracks = 30;

	while (ReadLine (fd, &hv0[0], sizeof (hv0)) > 0)
	{
	   if (hv0[0] == '#')
	      continue;

	   if ((p = strchr (hv0, '=')) == NULL) 
	      if ((p = strchr (hv0, ':')) == NULL)
	         continue;

	   *p = 0;
	   p++;

	   if (!*p)
	      continue;

	   if (!strncmp (hv0, "DTITLE", 6))
	      strncpy (cddb->title, ltrim (rtrim (p)), 80);

	   if (!strncmp (hv0, "TTITLE", 6))
	   {
	      i = atoi (&hv0[6]);

	      if (i >= 0 && i < cddb->tracks)
		 strncpy (cddb->entry[i].song, ltrim (rtrim (p)), 80);
	   }

	   if (!strncmp (hv0, "EXTT", 4))
	   {
	      i = atoi (&hv0[4]);

	      if (i >= 0 && i < cddb->tracks)
		 strncpy (cddb->entry[i].singer, ltrim (rtrim (p)), 80);
	   }
	}

	close (fd);

	/*
	 * Alle im CDDB-Eintrag vorhandenen Daten wurden kopiert. Nun
	 * kopieren wir noch die Spielzeiten und Frames in die Struktur.
	 */
	for (i = 0; i < cddb->tracks; i++)
	{
	   sprintf (cddb->entry[i].zeit, "%02d:%02d",
			disc.disc_track[i].track_length.minutes,
			disc.disc_track[i].track_length.seconds);
	   cddb->entry[i].frame = disc.disc_track[i].track_pos.frames;
	}

	return 0;
}

unsigned long cddbGetDiscID (struct disc_info *disc, char *device)
{
int fd;
int i, a, t, n;
unsigned long discid;

        t = n = 0;

	if (CheckLibVersion ())
	   return CD_LIBVERS;

        if ((fd = cd_init_device (device)) == -1)
	   return CD_EDEVICE;

        if (cd_stat (fd, disc) == -1)
        {
           cd_finish (fd);
           return CD_ESTATUS; 
        }

        cd_finish (fd);

        if (!disc->disc_present)
	   return CD_NOAUDIO;

	/*
	 * Ab hier beginnt die Berechnung der DiscID. Als Basis werden
         * die Gesamtanzahl Tracks verwendet. Dazu werden dann die Spiel-
         * minuten und Sekunden zusammen mit eine Pruefziffer summiert.
	 * Untenstehende Formel wurde aus 'xmcd' von Ti Kan extrahiert.
         */
        a = disc->disc_total_tracks;

        for (i = 0; i < a; i++)
           n += cddb_sum ((disc->disc_track[i].track_pos.minutes * 60) +
                disc->disc_track[i].track_pos.seconds);

        t = ((disc->disc_track[a].track_pos.minutes * 60) +
             disc->disc_track[a].track_pos.seconds) -
             ((disc->disc_track[0].track_pos.minutes * 60) +
             disc->disc_track[0].track_pos.seconds);

        discid = ((n % 0xff) << 24 | t << 8 | disc->disc_total_tracks);
        return discid;
}

/*
 * Pruefsummenberechnung
 */
int cddb_sum (int n)
{
int ret;

        for (ret = 0; n > 0; n /= 10)
                ret += n % 10;

        return (ret);
}

int cddbCreateCDDBEntry (char *path, cddb_title_t *cddb)
{
int fd, i;
char cdfile[256], hv0[256];

	sprintf (&cdfile[0], "%s/%08x", path, cddb->discid);

	if ((fd = open (cdfile, O_RDWR | O_CREAT | O_TRUNC, 0666)) == -1)
	{
	   fprintf (stderr, "Error opening or creating file %s!\n", cdfile);
	   return 1;
	}

	/*
	 * Da wir den CDDB EIntrag zum Schreiben oeffnen konnten,
	 * schreiben wir nun die Eintraege nacheinander in das File.
	 */
	strcpy (hv0, "# Record Management 0.6 CD database file\n");
	write (fd, hv0, strlen (hv0));
	strcpy (hv0, "# Copyright (C) 1998, 1999 by Andreas Theofilu\n");
	write (fd, hv0, strlen (hv0));
	strcpy (hv0, "#\n# Track frame offsets:\n#\n#\n");
	write (fd, hv0, strlen (hv0));
	sprintf (hv0, "DISCID=%08x\n", cddb->discid);
	write (fd, hv0, strlen (hv0));
	sprintf (hv0, "DTITLE=%s\n", cddb->title);
	write (fd, hv0, strlen (hv0));

	for (i = 0; i < cddb->tracks; i++)
	{
	   sprintf (hv0, "TTITLE%d=%s\n", i, cddb->entry[i].song);
	   write (fd, hv0, strlen (hv0));
	}

	strcpy (hv0, "EXTD=\n");
	write (fd, hv0, strlen (hv0));

	for (i = 0; i < cddb->tracks; i++)
	{
	   sprintf (hv0, "EXTT%d=%s\n", i, cddb->entry[i].singer);
	   write (fd, hv0, strlen (hv0));
	}

	strcpy (hv0, "PLAYORDER=\n");
	write (fd, hv0, strlen (hv0));
	close (fd);
	return 0;
}

int CheckLibVersion ()
{
char cdversion[20];
int Major, Minor, Patch, i;
char *p;

        cd_version (&cdversion[0], sizeof (cdversion));

	p = strtok (&cdversion[0], ".");
	i = 0;

	while (p)
	{
	   switch (i)
	   {
	      case 0: Major = atoi (p); break;
	      case 1: Minor = atoi (p); break;
	      case 2: Patch = atoi (p); break;
	   }

	   i++;
	   p = strtok (NULL, ".");
	}

	if (Major > 1)
	   return 1;

	if (Major != 0 || Minor != 99)
	   fprintf (stderr, "Warning! PLATTEN was made for libcdaudio 0.99.1!\n                 Hope it works with %d.%d.%d too.\n", Major, Minor, Patch);
}

/*
 * Folgende Funktion sucht in einem Unterverzeichnis <path> nach
 * Dateien die mit <filter> enden. In <anz> wird die Anzahl gefundener
 * Eintraege zurueckgegeben. Die Funktion gibt einen Zeiger auf den
 * ersten gefundenen Verzeichniseintrag zurueck. Jeder Eintrag hat
 * einen Verweis auf den naechsten.
 * Jeder Eintrag enthaelt den Songtitel, Interpreten und die
 * Spielzeit. Die Funktion liest den ID3Tag und den MP3-Header der
 * gefundenen Dateien aus. Wird kein MP3-Tag gefunden, wird der Titel
 * und der Artist aus dem Dateinamen generiert. Ein '-' gilt als
 * Trenner von Songtitel und Interpret, wobei der Interpret als erstes
 * im Dateinamen angenommen wird.
 */
dirlist_t *cddbGetDirList (char *path, int *anz, char *filter)
{
struct dirent *dp;
DIR *dirp;
char *p;
dirlist_t *first, *akt;
struct stat buf;
char hv0[1024];
int flag = 0;

	first = akt = NULL;

	if ((dirp = opendir (path)) == NULL)
	{
	   fprintf (stderr, "Error opening directory <%s>!\n", path);
	   return NULL;
	}

	while ((dp = readdir (dirp)) != NULL)
	{
	   flag = 0;

	   if (dp->d_name[0] == '.')
	      continue;

	   sprintf (hv0, "%s/%s", path, dp->d_name);
	   stat (hv0, &buf);

	   if (S_ISDIR (buf.st_mode))
	      continue;

	   if (filter)
	   {
	      if ((p = strrchr (dp->d_name, '.')) == NULL)
	         continue;

	      p++;

	      if (*p && !strcasecmp (p, filter))
	      {
	         if ((akt = AddEntry (akt, dp->d_name)) == NULL)
		 {
		    closedir (dirp);
		    return NULL;
		 }

		 flag = 1;
	      }
	   }
	   else
	   {
	      if ((akt = AddEntry (akt, dp->d_name)) == NULL)
	      {
	         closedir (dirp);
	         return NULL;
	      }

	      flag = 1;
	   }

	   if (flag)
	   {
	      if (!first)
	         first = akt;

	      GetID3Tag (hv0, &akt->title[0], &akt->artist[0], &akt->minutes,
			&akt->seconds, buf.st_size);

	      *anz = *anz + 1;
	   }
	}

	closedir (dirp);
	return first;
}

/*
 * Die folgende Funktion haengt einen neuen Eintrag an die
 * Directorykette. Existiert noch kein Eintrag, wird ein neuer
 * erzeugt.
 */
dirlist_t *AddEntry (dirlist_t *pos, char *d_name)
{
dirlist_t *first, *akt;

	akt = pos;

	if (!akt)
	{
	   if ((first = (dirlist_t *)malloc (sizeof (dirlist_t))) == NULL)
	   {
	      fprintf (stderr, "Not enough memory for directory entry!\n");
	      return NULL;
	   }

	   memset (first, 0, sizeof (dirlist_t));
	   first->dname = strdup (d_name);
	   first->next = NULL;
	   akt = first;
	}
	else
	{
	   dirlist_t *tmp;

	   if ((tmp = (dirlist_t *)malloc (sizeof (dirlist_t))) == NULL)
	   {
	      fprintf (stderr, "Not enough memory for directory entry!\n");
	      FreeDirList (first);
	      return NULL;
	   }

	   memset (tmp, 0, sizeof (dirlist_t));
	   tmp->dname = strdup (d_name);
	   tmp->next = NULL;
	   akt->next = tmp;
	   akt = tmp;
	}

	return akt;
}

/*
 * Folgende Funktion gibt die im Speicher allokierte Verzeichniskette
 * wieder frei.
 */
void FreeDirList (dirlist_t *first)
{
dirlist_t *p;

	p = first;

	while (p)
	{
	   p = p->next;
	   free (first->dname);
	   free (first);
	   first = p;
	}
}

/*
 * Folgende Funktion oeffnet eine angegebene MP3-Datei und liest den
 * ID3-Tag sowie den MP3-Header aus. Die Daten werden in unsere
 * Struktur eingetragen.
 */
int GetID3Tag (char *path, char *title, char *artist, int *minutes,
		int *seconds, unsigned long fileSize)
{
int fd, readsize, i;
tag song;
mp3header_t mh;
unsigned char buffer[70];
unsigned char *p;
int temp;

	if ((fd = open (path, O_RDONLY, 0400)) == -1)
	{
	   fprintf (stderr, "Konnte Datei %s nicht oeffnen!\n", path);
	   return -1;
	}

	/*
	 * Als erstes lesen wir den MP3-Tag ein, welcher sich am Ende
	 * jeder MP3-Datei befindet und 128 Zeichen lang ist.
	 */
	lseek (fd, -128L, SEEK_END);
	read (fd, &song, sizeof (tag));

	/*
	 * Nun lesen wir den MP3-Header ein. Dieser befindet sich in
	 * den ersten 1024 Bytes. Die einzelnen Bit muessen
	 * entsprechend extrahiert werden.
	 */
	lseek (fd, 0L, SEEK_SET);
	readsize = read (fd, &buffer[0], sizeof (buffer));

	if (readsize >= sizeof (buffer))
	{
	   i = temp = 0;

	   /*
	    * Normalerweise befindet sich der MP3-Header am Anfang der
	    * Datei. Das muss jedoch nicht sein. Der Header muss sich
	    * jedoch innerhalb der ersten 70 Bytes befinden!
	    * Der ganze Header besteht nur aus 4 Bytes. Eingeleitet
	    * wird er immer durch 0xfffx. Das 'x' steht fuer 0 - e.
	    */
	   while (i < (readsize - 4))
	   {
	      if (buffer[i] == 0xff)
	      {
		 if (buffer[i+1] >= 0xf0 && buffer[i+1] <= 0xfe)
	         {
		    temp = 1;
		    break;
		 }
	      }

	      i++;
	   }

	   /*
	    * Finden wir innerhalb der ersten 70 Bytes keinen Header,
	    * ist die Datei fehlerhaft oder keine MP3-Datei.
	    */
	   if (!temp)
	   {
	      close (fd);
	      return -1;
	   }

	   mh.version = get1bit (buffer[i+1], 4);
	   mh.lay = 4 - getbits (buffer[i+1], 3, 2);
	   mh.error_protection = !get1bit (buffer[i+1], 1);
	   mh.bitrate_index = getbits (buffer[i+2], 8, 4);
	   mh.sampling_frequency = getbits (buffer[i+2], 4, 2);
	   mh.padding = get1bit (buffer[i+2], 2);
	   mh.extension = get1bit (buffer[i+2], 1);
	   mh.mode = getbits(buffer[i+3], 8, 2);
	   mh.mode_ext = getbits(buffer[i+3], 6, 2);
	   mh.copyright = get1bit (buffer[i+3], 4);
	   mh.original = get1bit (buffer[i+3], 3);
	   mh.emphasis = getbits (buffer[i+3], 2, 2);
	   mh.stereo = (mh.mode == 3) ? 1 : 2;
 	   mh.pcm = 32;

	   if (mh.lay == 3)
	   {
	      mh.pcm *= 18;

	      if (mh.version == 0)
	         mh.pcm *= 2;
	   }
	   else
	   {
	      mh.pcm *= 12;

	      if (mh.lay == 2)
		 mh.pcm *= 3;
	   }

	   /*
	    * Zum Berechnen der Spielzeit benoetigen wir nur einen
	    * Bruchteil der oben gewonnen Daten. Die Zeit wird dann
	    * wie folgt berechnet:
	    *    seconds = FileSize / ((bits per sec) / 8)
	    */
	   *seconds = fileSize / ((bitrates[0][mh.version][mh.bitrate_index-1] * 1000L) / 8L);
	   *minutes = *seconds / 60;
	   *seconds = *seconds % 60;
	}

	close (fd);

	/*
	 * Einfuegen der gewuenschten Daten in unsere eigene Struktur
	 */
	if (!strncmp (song.tag, "TAG", 3))
	{
	   strncpy (title, song.title, 30);	title[30] = 0;
	   strncpy (artist, song.artist, 30);	artist[30] = 0;
	}
	else
	{
	   char *dpath, *dp;

	   dpath = strdup (path);

	   if ((p = strrchr (dpath, '/')) != NULL)
	      p++;
	   else
	      p = dpath;

	   if ((dp = strrchr (p, '.')) != NULL)
	   {
	      *dp = 0;

	      if ((dp = strchr (p, '-')) != NULL)
	      {
	         *dp = 0;
		 strncpy (title, dp+1, 30);	title[30] = 0;
		 strncpy (artist, p, 30);	artist[30] = 0;
	      }
	      else
	      {
	         strncpy (title, p, 30);
		 title[30] = 0;
	      }
	   }
	   else
	   {
	      strncpy (title, p, 30);
	      title[30] = 0;
	   }

	   free (dpath);
	}

	return (0);
}

/*
 * Folgende Funktion liefert genau den Wert eines Bits and der
 * angebenen Position <pos>. Es wird von 1 weg gezaehlt.
 */
int get1bit (unsigned char byte, int pos)
{
unsigned int bit;
int i;

	if (pos > 8 || pos < 1)
	   return -1;

	i = pos - 1;
	bit = (byte & mask[i]) >> (i);
	return (int)bit;
}

/*
 * Folgende Funktion liefert den Wert von <anz> Bits innerhalb eines
 * Bytes, beginnend von <start>.
 */
int getbits (unsigned char byte, int start, int anz)
{
unsigned int bits;
unsigned int bm;

	if (start < anz)
	   return -1;

	if (anz < 1)
	   return -1;

	switch (anz)
	{
	   case 1: bm = 0x0001; break;
	   case 2: bm = 0x0003; break;
	   case 3: bm = 0x0007; break;
	   case 4: bm = 0x000f; break;
	   case 5: bm = 0x001f; break;
	   case 6: bm = 0x003f; break;
	   case 7: bm = 0x007f; break;
	   case 8: bm = 0x00ff; break;
	}

	bits = (unsigned int)(byte >> (start - anz)) & bm; 
	return (int)bits;
}
