/*
 * File   : platten.c
 * Purpose: This is the main file of the application platten and
 *          initializes all the resources used by the application.
 * Autor  : Andreas Theofilu
 *
 * Copyright (C) 1998, 1999 by Theos Soft.
 */

/* 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <Xmt/Xmt.h>
#include <Xmt/Create.h>
#include <Xmt/Converters.h>
#include <Xmt/Procedures.h>
#include <Xmt/Layout.h>
#include <Xmt/Xpm.h>
#include <Xmt/Dialog.h>
#include <Xmt/InputField.h>
#include <Xmt/Symbols.h>
#include <Xbae/Matrix.h>
#include "platten.h"

#include "platten.xbm"
#include "platten.xpm"

/*
 * Global used Variables
 */
Widget toplevel;
application_resource_t app_resources;
static char AboutMessage[512];

/*
 * Some resources used by the program
 */
static XtResource ProgValues[] = {
	{ "Database", "Database", XmRString,
	   sizeof (String),
	   XtOffset (application_resource_t*, Database),
	   XmRString, "INFORMIX" },
	{ "Version", "Version", XmRString,
	  sizeof (String),
	  XtOffset (application_resource_t*, Version),
	  XmRString, "Unknown" },
	{ "datePattern", "datePattern", XmRString,
	  sizeof (String),
	  XtOffset (application_resource_t*, datePattern),
	  XmRString, "dd.mm.yyyy" },
	{ "CDDBPath", "CDDBPath", XmRString,
	  sizeof (String),
	  XtOffset (application_resource_t*, CDDBPath),
	  XmRString, "/usr/X11/lib/X11/xmcd/cddb" },
	{ "cdrom", "cdrom", XmRString,
	  sizeof (String),
	  XtOffset (application_resource_t*, cdrom),
	  XmRString, "/dev/cdrom" },
	{ "cdmount", "cdmount", XmRString,
	  sizeof (String),
	  XtOffset (application_resource_t*, cdmount),
	  XmRString, "/cdrom" }
};

static XmtProcedureInfo xt_procedures[] = {
	{"FirstFillCallback", (XmtProcedure)FirstFillCallback,
	 {XtRWidget, XtRWidget}},
	{"rclInitChooser", (XmtProcedure)rclInitChooser,
	 {XtRWidget, XtRWidget}},
	{"OptionDialogInit", (XmtProcedure)OptionDialogInit,
	 {XtRWidget}},
	{"UpdrclInitDiscid", (XmtProcedure)UpdrclInitDiscid,
	 {XtRWidget}}
};

/*
 * Prototype function calls in this file not used in any other file.
 */
static void About ();
void Quit ();

/*
 * Entry point and start of the program :-).
 */
main (int argc, char *argv[])
{
XtAppContext app;
XmtImage *platten_image;
Pixmap iconPixmap;
Arg arglist[10];
int n = 0;

	XtSetArg (arglist[n], XmNiconName, "Platten"); n++;
	XtSetArg (arglist[n], XmNminHeight, 200); n++;
	XtSetArg (arglist[n], XmNminWidth, 400); n++;
	XtSetArg (arglist[n], XmNbaseHeight, 200); n++;
	XtSetArg (arglist[n], XmNbaseWidth, 400); n++;
	XtSetArg (arglist[n], XmNmaxHeight, 200); n++;
	XtSetArg (arglist[n], XmNmaxWidth, 400); n++;

	toplevel = XmtInitialize (&app, "Platten", NULL, 0, &argc, argv,
				  NULL, arglist, n);

	iconPixmap = XCreateBitmapFromData (XtDisplay (toplevel),
			XtScreen (toplevel)->root, platten_bits,
			platten_width, platten_height);
	XtVaSetValues (toplevel, XmNiconPixmap, iconPixmap, NULL);

	XmtRegisterWidgetClass ("XbaeMatrix", xbaeMatrixWidgetClass);
	XtGetApplicationResources (toplevel, (XtPointer)&app_resources,
			ProgValues, XtNumber (ProgValues), NULL, 0);

	if (!app_resources.Version || strcmp (app_resources.Version, "0.6"))
	{
	   fprintf (stderr, "The resource file is for a different version (%s) of this program\nor even for another program!\nThis program will not work without the correct resource file.\n", app_resources.Version);
	   exit (1);
	}

	if (!app_resources.Database || strcmp (app_resources.Database, "INFORMIX"))
	{
	   fprintf (stderr, "This program can work with the database Informix only!\nThe defined database %s is not supported currently.\n", app_resources.Database);
	   exit (1);
	}

	XmtRegisterAll ();
	XmtRegisterImprovedIcons (toplevel, NULL);
	XmtRegisterProcedures (xt_procedures, XtNumber (xt_procedures));

	XmtRegisterCallbackProcedure ("NewSingle", NewSingle, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("SearchSingle", SearchSingle, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("About", About, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("Quit", Quit, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("CheckDateCallback",
				CheckDateCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("AddNewSingleCallback",
				AddNewSingleCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("ClearValuesCallback",
				ClearValuesCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("QuerySingleCallback",
				QuerySingleCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("NewSingleCallback",
				NewSingleCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("NextSingleCallback",
				NextSingleCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("PreviousSingleCallback",
				PreviousSingleCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("UpdateSingleCallback",
				UpdateSingleCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("OkaySingleCallback",
				OkaySingleCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("SingleValueChangedCallback",
				SingleValueChangedCallback, XmtRCallbackData);
	
	XmtRegisterCallbackProcedure ("NewLP", NewLP, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("NewLPCut", NewLPCut, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("NewLPCutDialog", NewLPCutDialog, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("UpdateLpCallback",
				UpdateLpCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("AddLpLabelCallback",
				AddLpLabelCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("AddLpCutsCallback",
				AddLpCutsCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("rclClearValuesCallback",
				rclClearValuesCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("rccClearValuesCallback",
				rccClearValuesCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("rclAutoCallback",
				rclAutoCallback, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("SearchLps", SearchLps, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("LPSearchValueChangedCallback",
				LPSearchValueChangedCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("MasterListCallback",
				MasterListCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("lpsPartChooserCallback",
				lpsPartChooserCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("lpsSearchTypeCallback",
				lpsSearchTypeCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("lpsEnterCellCallback",
				lpsEnterCellCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("lpsCutMasterCallback",
				lpsCutMasterCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("NewKennChangedCallback",
				NewKennChangedCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("UpdateKennChangedCallback",
				UpdateKennChangedCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("GetCDDBCallback",
				GetCDDBCallback, XmtRCallbackData);
	XmtRegisterCallbackProcedure ("PutCDDBCallback",
				PutCDDBCallback, XmtRCallbackData);

	XmtRegisterCallbackProcedure ("PrintSingles", PrintSingles, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("PrintLps", PrintLps, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("StopPrintCallback",
				StopPrintCallback, XmtRCallbackData);

	XmtRegisterCallbackProcedure ("Options", Options, XmtRCallbackUnused);
	XmtRegisterCallbackProcedure ("OptionChooserChangedCallback",
				OptionChooserChangedCallback, XmtRCallbackData);

	/*
	 * Registering the application icon for later use in dialogs.
	 */
	platten_image = XmtParseXpmData (platten);
	XmtRegisterImage ("platten", platten_image);

	/*
	 * Creating the Children of the main window and display all together.
	 */
	XmtCreateChildren (toplevel);
	XtRealizeWidget (toplevel);

	/*
	 * Initializing the program and the database.
	 */
	XmtDisplayBusyCursor (toplevel);

	if (!GetOptions (toplevel))
	   return;

	init ();
	XmtDisplayDefaultCursor (toplevel);
	XtAppMainLoop (app);
}

static void About ()
{
static Widget w = NULL;

	if (!w)
	   w = XmtBuildDialog (toplevel, "about", NULL, 0);
	else
	   XtManageChild (w);
}

void Quit ()
{
	CloseDatabase ();
	exit (0);
}

/*
 * Some Callbacks used by various dialog boxes
 */
void CheckDateCallback (Widget w, XtPointer client_data, XtPointer call_data)
{
int ret;
XmtInputFieldCallbackStruct *data = (XmtInputFieldCallbackStruct *)call_data;

	ret = CheckDate (w, (char *)data->input);

	if (ret != 0)
	   data->okay = False;
	else
	   data->okay = True;
}
